/*
===============================================================================
 Name        : acl810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "mrt.h"
#include "i2cm.h"
#include "adxl345.h"

// TODO: insert other definitions and declarations here
#define ACL_TOLERANT 4

inline void pwmDutySetup(uint8_t index, uint8_t duty){
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, index, Chip_SCTPWM_PercentageToTicks(LPC_SCT, duty));
}

void setRpm(int motor, int slope){
	int index;
	uint8_t duty;

	index =  (motor & 1) * 2 + 1;
	if(slope < 0){
		pwmDutySetup(index, 0);
		slope *= -1;
		index++;
	} else
		pwmDutySetup(index + 1, 0);

	if(slope < ACL_TOLERANT){
		pwmDutySetup(index, 0);
	} else
	if(slope >= 224){
		pwmDutySetup(index, 100);
	} else {
		duty = (slope / 32) * 10 + 35;
		pwmDutySetup(index, duty);
	}
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
    SystemCoreClockUpdate();

    Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);//Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);//Disable SWDIO
   	Chip_SWM_DisableFixedPin(SWM_FIXED_RST);//Disable RESET
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 0);//I2C_SDA
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);//I2C_SCL
   	Chip_SWM_MovablePinAssign(SWM_CTOUT_0_O, 5);
   	Chip_SWM_MovablePinAssign(SWM_CTOUT_1_O, 4);
   	Chip_SWM_MovablePinAssign(SWM_CTOUT_2_O, 2);
   	Chip_SWM_MovablePinAssign(SWM_CTOUT_3_O, 3);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	i2cmSetup();
	aclSetup();
	mrtSetup();

	Chip_SCTPWM_Init(LPC_SCT);
	Chip_SCTPWM_SetRate(LPC_SCT, 20000);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 1, 0);
	Chip_SCTPWM_SetOutPin(LPC_SCT, 1, 0);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 2, 0);
	Chip_SCTPWM_SetOutPin(LPC_SCT, 2, 1);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 3, 0);
	Chip_SCTPWM_SetOutPin(LPC_SCT, 3, 2);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 4, 0);
	Chip_SCTPWM_SetOutPin(LPC_SCT, 4, 3);
	Chip_SCTPWM_Start(LPC_SCT);

	while(1){
		int16_t x, y, z;

		aclReadAxis(&x, &y, &z);
		setRpm(0, x);
		setRpm(1, y);
		mrtWait(MRT_MS(200));
	}
    return 0 ;
}
