/*
 * i2cm.c
 *
 *  Created on: 2014/10/09
 *      Author: Tetsuya Suzuki
 */

#include "chip.h"

I2C_HANDLE_T *i2cmHandle;
uint32_t i2cmMem[0x20];
void (*i2cmError)(uint8_t adrs, ErrorCode_t err) = NULL;

void i2cmSetupErr(void (*pFunc)(uint8_t, ErrorCode_t)){
	i2cmError = pFunc;
}

void i2cmSetup(){
	Chip_I2C_Init(LPC_I2C);
	i2cmHandle = LPC_I2CD_API->i2c_setup(LPC_I2C_BASE, i2cmMem);
	if(i2cmHandle == NULL){
		if(i2cmError == NULL)
			while(1);
		else
			i2cmError(0, 1);
	}

	ErrorCode_t err = LPC_I2CD_API->i2c_set_bitrate(i2cmHandle, SystemCoreClock, 100000);
	if (err != LPC_OK){
		if(i2cmError == NULL)
			while(1);
		else
			i2cmError(0, err);
	}

	LPC_I2CD_API->i2c_set_timeout(i2cmHandle, 100000);
}

void i2cmRx(uint8_t *buf, uint8_t count){
	I2C_PARAM_T par;
	I2C_RESULT_T res;
	ErrorCode_t err;

	par.num_bytes_send = 0;
	par.num_bytes_rec = count;
	par.buffer_ptr_rec = buf;
	par.stop_flag = 1;

	err = LPC_I2CD_API->i2c_master_receive_poll(i2cmHandle, &par, &res);

	if (err != LPC_OK){
		if(i2cmError == NULL)
			while(1);
		else
			i2cmError(*buf, err);
	}
}

void i2cmTx(uint8_t *buf, uint8_t count){
	I2C_PARAM_T par;
	I2C_RESULT_T res;
	ErrorCode_t err;

	par.num_bytes_send = count;
	par.buffer_ptr_send = buf;
	par.num_bytes_rec = 0;
	par.stop_flag = 1;

	err = LPC_I2CD_API->i2c_master_transmit_poll(i2cmHandle, &par, &res);

	if (err != LPC_OK){
		if(i2cmError == NULL)
			while(1);
		else
			i2cmError(*buf, err);
	}
}
