/*
 ===============================================================================
 Name        : fbc810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : finger top beat counter
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "mrt.h"
#include "uart.h"
#include "form.h"

// TODO: insert other definitions and declarations here
#define FBC_VLADDR 20

volatile int beat = -1;

void buzOff() {
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);
}

void SCT_IRQHandler(void) {
	uint32_t period;

	period = LPC_SCT->CAP[0].U;
	beat = SystemCoreClock * 60 / period;

	Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 3);
	mrtOneshot(MRT_MS(50), buzOff);

	Chip_SCT_ClearEventFlag(LPC_SCT, SCT_EVT_0);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO0, PIN_MODE_INACTIVE);   //ACMP_I1
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);
	Chip_SWM_EnableFixedPin(SWM_FIXED_ACMP_I1); //ACMP_I1
	Chip_SWM_MovablePinAssign(SWM_ACMP_O_O, 2); //LED
	Chip_SWM_MovablePinAssign(SWM_CTIN_0_I, 2); //CTIN_0
	Chip_SWM_MovablePinAssign(SWM_U0_TXD_O, 4);
	Chip_SWM_MovablePinAssign(SWM_U0_RXD_I, 0);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 3);
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);

	//SCT setup
	Chip_SCT_Init(LPC_SCT);
	Chip_SCT_Config(LPC_SCT, SCT_CONFIG_32BIT_COUNTER);
	LPC_SCT->REGMODE_U |= (1 << 0);
	LPC_SCT->EV[0].CTRL |= (1 << 10) | (2 << 12);
	LPC_SCT->EV[0].STATE = 3;
	LPC_SCT->CAPCTRL[0].U = SCT_EVT_0;
	LPC_SCT->LIMIT_U |= SCT_EVT_0;
	Chip_SCT_EnableEventInt(LPC_SCT, SCT_EVT_0);
	NVIC_EnableIRQ(SCT_IRQn);
	Chip_SCT_ClearControl(LPC_SCT, SCT_CTRL_HALT_L);

	mrtSetup();
	uartSetup();

	Chip_ACMP_Init(LPC_CMP);
	Chip_ACMP_SetupVoltLadder(LPC_CMP, FBC_VLADDR, false);
	Chip_ACMP_EnableVoltLadder(LPC_CMP);
	mrtWait(MRT_US(30));

	Chip_ACMP_SetPosVoltRef(LPC_CMP, ACMP_POSIN_ACMP_I1);
	Chip_ACMP_SetNegVoltRef(LPC_CMP, ACMP_NEGIN_VLO);
	Chip_ACMP_SetHysteresis(LPC_CMP, ACMP_HYS_10MV);

	while (1) {
		__WFI();

		if (beat > 0) {
			uartPuts("Beat: ");
			uartPuts(formItoa(beat));
			uartPuts("\r\n");
			beat = -1;
		}
	}
	return 0;
}
