/*
===============================================================================
 Name        : fbc810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : finger top beat counter
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "i2cs.h"
#include "mrt.h"

// TODO: insert other definitions and declarations here
#define I2C_ADDR (0x26)
#define FBC_VLADDR 20

void buzOff(){
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);
}

void SCT_IRQHandler(void){
	uint32_t period;

	period = LPC_SCT->CAP[0].U;
	i2csTxBuf[0] = SystemCoreClock * 60 / period;

	if((i2csRxBuf[0] & 0x80) == 0){
		Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 3);
		mrtOneshot(MRT_MS(50), buzOff);
	}

	Chip_SCT_ClearEventFlag(LPC_SCT, SCT_EVT_0);
}

void changeLad(uint8_t rxn, uint8_t txn){
	if(rxn && (txn == 0)){
		Chip_ACMP_SetupVoltLadder(LPC_CMP, i2csRxBuf[0], false);
		Chip_ACMP_EnableVoltLadder(LPC_CMP);
	}
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO0, PIN_MODE_INACTIVE);//ACMP_I1
	//Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO2, PIN_MODE_INACTIVE);//LED
	//Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO3, PIN_MODE_INACTIVE);//Buz
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

   	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
   	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);//Disable SWCLK
   	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);//Disable SWDIO
   	Chip_SWM_EnableFixedPin(SWM_FIXED_ACMP_I1);//ACMP_I1
 	Chip_SWM_MovablePinAssign(SWM_ACMP_O_O, 2);//LED
	Chip_SWM_MovablePinAssign(SWM_CTIN_0_I, 2);//CTIN_0
	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 4);//I2C_SDA
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);//I2C_SCL
   	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 3);
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);

	//SCT setup
	Chip_SCT_Init(LPC_SCT);//clocks and reset
	Chip_SCT_Config(LPC_SCT, SCT_CONFIG_32BIT_COUNTER);
	LPC_SCT->REGMODE_U |= (1 << 0);
	LPC_SCT->EV[0].CTRL |= (1 << 10) | (2 << 12);//I/O condition rise
	LPC_SCT->EV[0].STATE = 3; /*< Trigger event at any state */
	LPC_SCT->CAPCTRL[0].U = SCT_EVT_0; //Event0, capture0
	LPC_SCT->LIMIT_U |= SCT_EVT_0;//Clear at event0
    Chip_SCT_EnableEventInt(LPC_SCT, SCT_EVT_0);
    NVIC_EnableIRQ(SCT_IRQn); // enable SCT interrupt
	Chip_SCT_ClearControl(LPC_SCT, SCT_CTRL_HALT_L);// start

	mrtSetup();

	// Init analog comparator
   	Chip_ACMP_Init(LPC_CMP);
	Chip_ACMP_SetupVoltLadder(LPC_CMP, FBC_VLADDR, false);
	Chip_ACMP_EnableVoltLadder(LPC_CMP);
	mrtWait(MRT_US(30));//30us power-up settling time

	Chip_ACMP_SetPosVoltRef(LPC_CMP, ACMP_POSIN_ACMP_I1);
	Chip_ACMP_SetNegVoltRef(LPC_CMP, ACMP_NEGIN_VLO);
	Chip_ACMP_SetHysteresis(LPC_CMP, ACMP_HYS_10MV);
	//Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_ACOMP);

	i2csSetupCallback(changeLad);
	i2csSetup(I2C_ADDR, 1, 1);

	while(1) __WFI();
	return 0;
}
