/*
===============================================================================
 Name        : frq810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "romi2cs.h"
#include "mrt.h"

// TODO: insert other definitions and declarations here
#define I2C_ADDR (0x24 << 1)

void alarm(){
	Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 2);
	while(1);
}

void changeInterval(){
	Chip_MRT_SetInterval(LPC_MRT_CH(0), SystemCoreClock / (uint32_t)(i2csRxBuf[1] + 1));
}

void getFreq(){
	uint32_t freq;

	Chip_SCT_SetControl(LPC_SCT, SCT_CTRL_HALT_L);// stop
	freq = LPC_SCT->COUNT_U;
	LPC_SCT->COUNT_U = 0;
	Chip_SCT_ClearControl(LPC_SCT, SCT_CTRL_HALT_L);// start
	__disable_irq();
	i2csTxBuf[0] = (freq >> 24);
	i2csTxBuf[1] = (freq >> 16) & 0xff;
	i2csTxBuf[2] = (freq >> 8) & 0xff;
	i2csTxBuf[3] = freq & 0xff;
	__enable_irq();

	Chip_GPIO_SetPinToggle(LPC_GPIO_PORT, 0, 3);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
	SystemCoreClockUpdate();

	//Pin assign
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 0);
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);
	Chip_SWM_MovablePinAssign(SWM_CTIN_0_I, 4);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 2);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 3);
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 2);
	Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 3);

	//SCT setup
	Chip_SCT_Init(LPC_SCT);//clocks and reset
	Chip_SCT_Config(LPC_SCT, SCT_CONFIG_32BIT_COUNTER | SCT_CONFIG_CLKMODE_INCLK);
	Chip_SCT_ClearControl(LPC_SCT, SCT_CTRL_HALT_L);// start

	//MRT setup
	mrtSetup();
	mrtRepeat(SystemCoreClock, getFreq);
	mrtRepeatStart();

	//I2C setup
	i2csSetupErr(alarm);
	i2csSetupRx(changeInterval);
	i2csSetup(I2C_ADDR, 2, 4);
	//i2csSetup(I2C_ADDR, 5, 4);

	while(1) __WFI();
    return 0 ;
}
