/*
 ===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "uart.h"
#include "form.h"
#include "i2cm.h"
#include "mrt.h"

// TODO: insert other definitions and declarations here
#define HUM_ADRS (0x24 << 1)

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);    // Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);    // Disable SWDIO
	Chip_SWM_MovablePinAssign(SWM_U0_TXD_O, 4);
	Chip_SWM_MovablePinAssign(SWM_U0_RXD_I, 0);
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 2);// I2C_SDA for DEBUG
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 3);// I2C_SCL for DEBUG
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	i2cmSetup();
	uartSetup();
	mrtSetup();

	uint8_t lange = 0;

	uartPuts("hum810 write/read details.\r\n");
	while (1) {
		uint8_t i2cbuf[5];
		uint32_t freq;

		i2cbuf[0] = HUM_ADRS;
		i2cbuf[1] = lange;
		i2cmTx(i2cbuf, 2);
		mrtWait(MRT_MS(2000));

		i2cmRx(i2cbuf, 5);
		uartPuts("Lange=");
		uartPuts(formItoa(lange));
		uartPuts(", Freq=");
		freq =
			((uint32_t)i2cbuf[1] << 24) +
			((uint32_t)i2cbuf[2] << 16) +
			((uint32_t)i2cbuf[3] << 8) +
			(uint32_t)i2cbuf[4];
		uartPuts(formItoa(freq));
		uartPuts("\r\n");

		lange++;
		if(lange > 2)
			lange = 0;
	}
	return 0;
}
