/*
 ===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "mrt.h"
#include "uart.h"
#include "form.h"
#include "remocon.h"

// TODO: insert other definitions and declarations here
#define RC_US(clks) (clks / (RC_CLKS_S / 1000000))

void report(irform_t *ir) {
	int i;

	uartPuts("Format: ");
	uartPuts(ir->lon < RC_FORM ? "IEHA, " : "NEC, ");
	uartPuts(ir->multi ? "multi" : "single");
	uartPuts(" shot.\r\nLeader ON:");
	uartPuts(formItoa(RC_US(ir->lon)));
	uartPuts(", OFF:");
	uartPuts(formItoa(RC_US(ir->loff)));
	uartPuts("\r\nSignal T:");
	uartPuts(formItoa(RC_US(ir->t)));
	uartPuts(", H:");
	uartPuts(formItoa(RC_US(ir->h)));
	uartPuts(", L:");
	uartPuts(formItoa(RC_US(ir->l)));
	uartPuts("\r\n");

	if (ir->multi)
		uartPuts("1st shot ");

	uartPuts(formItoa(ir->count1));
	uartPuts(" signals\r\n");
	for (i = 0; i < ir->count1 - 1; i++) {
		uartPuts(formHex(ir->code1[i], 2));
		uartPuts((i + 1) % 8 ? ", " : "\r\n");
	}
	uartPuts(formHex(ir->code1[i], 2));
	uartPuts("\r\n");

	if (!ir->multi)
		return;

	uartPuts("Gap:");
	uartPuts(formItoa(RC_US(ir->gap)));

	uartPuts("\r\n2nd shot ");
	uartPuts(formItoa(ir->count2));
	uartPuts(" signals\r\n");
	for (i = 0; i < ir->count2 - 1; i++) {
		uartPuts(formHex(ir->code2[i], 2));
		uartPuts((i + 1) % 8 ? ", " : "\r\n");
	}
	uartPuts(formHex(ir->code2[i], 2));
	uartPuts("\r\n");
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	uint8_t code1[RC_SIGLEN], code2[RC_SIGLEN];
	irform_t ir;

	ir.code1 = code1;
	ir.code2 = code2;

	bool analyzed = false;
	static const char *err_msg[] = {
		"Time out.",
		"Illegal format.",
		"Too long.",
		"Bit lacked."
	};

	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);
	Chip_SWM_MovablePinAssign(SWM_U0_TXD_O, 4);
	Chip_SWM_MovablePinAssign(SWM_U0_RXD_I, 0);
   	Chip_SWM_MovablePinAssign(SWM_CTOUT_0_O, 3);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	uartSetup();
	mrtSetup();
	mrtSetupSleep();
	rc_setup(2);

	uartPuts("IR remocon analyzer\r\n");

	while (1) {
		uint8_t c;
		int ret;

		uartPuts("\r\nSelect [A]nalyze, [T]ransmit ");
		c = uartGetch();
		uartPutch(c);
		uartPuts("\r\n");

		switch (c) {
		case 'A':
		case 'a':
			uartPuts("Ready.\r\n");
			ret = rc_receive(&ir);
			if (ret == RC_ERR_NORMAL){
				analyzed = true;
				report(&ir);
			} else {
				analyzed = false;
				uartPuts("Error:");
				uartPuts(err_msg[ret + 4]);
				uartPuts("\r\n");
			}
			break;
		case 'T':
		case 't':
			if(!analyzed){
				uartPuts("Not yet analyzed.\r\n");
			} else {
				rc_transmit(&ir);
				uartPuts("Transmit done.\r\n");
			}
			break;
		}
	}
	return 0;
}
