/*
 ===============================================================================
 Name        : irr810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "mrt.h"
#include "remocon.h"

#define SRV_CENTER MRT_US(1500)
#define SRV_STEP MRT_US(100)

// TODO: insert other definitions and declarations here
volatile uint32_t pulseWidth;

void pulseStop() {
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);
}

void pulseStart() {
	Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 3);
	mrtOneshot(pulseWidth, pulseStop);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	uint8_t code1[RC_SIGLEN], code2[RC_SIGLEN]; // array byte code
	irform_t ir1;

	ir1.code1 = code1;
	ir1.code2 = code2;

	//Sharp AQUOS
	irform_t ir2;
	static uint8_t data[3][6] = {
		{ 0x55, 0x5a, 0xf1, 0x48, 0x1b, 0x8f },
		{ 0x55, 0x5a, 0xf1, 0x48, 0xeb, 0x80 },
		{ 0x55, 0x5a, 0xf1, 0x48, 0x4a, 0x8b }
	};
	ir2.count1 = 6;
	ir2.multi = false;

/*
	//TMY TLD-RNT06P
	static uint8_t data[3][4] = {
		{ 0x61, 0xbf, 0x24, 0xdb },//right
		{ 0x61, 0xbf, 0x64, 0x9b },//center
		{ 0x61, 0xbf, 0xa4, 0x5b } //left
	};
	ir2.count1 = 4;
	ir2.multi = false;
*/

	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);    // Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);    // Disable SWDIO
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO3, PIN_MODE_INACTIVE);
	Chip_IOCON_PinEnableOpenDrainMode(LPC_IOCON, IOCON_PIO3);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	//GPIO setup
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 3);

	rc_setup(2);
	mrtSetup();
	mrtSetupSleep();
	pulseWidth =  MRT_US(2000);
	mrtRepeat(MRT_MS(20), pulseStart);
	mrtSleep(MRT_MS(1000));
	pulseWidth = MRT_US(1000);
	mrtSleep(MRT_MS(1000));
	pulseWidth = SRV_CENTER;

	while (1) {
		int i;

		while (rc_receive(&ir1));
		for(i = 0; i < 3; i++){
			ir2.code1 = data[i];
			if(rc_ircmp(&ir1, &ir2) == 0){
				switch(i){
				case 0:
					pulseWidth += SRV_STEP;
					break;
				case 1:
					pulseWidth = SRV_CENTER;
					break;
				case 2:
					pulseWidth -= SRV_STEP;
					break;
				}
				break;
			}
		}
	}
	return 0;
}
