/*
 ===============================================================================
 Name        : suc810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
// TODO: insert other definitions and declarations here
#define SUC_FREQ 24000
#define SUC_STOP 1000
#define SUC_SKIP 10
#define SUC_TERM (SystemCoreClock / SUC_FREQ)
#define SUC_LIMIT (SUC_TERM * 80 / 100)//80%

void SCT_IRQHandler(void) {
	uint32_t duty;
	static uint32_t stop = 0;
	static uint32_t skip = 0;

	Chip_SCT_ClearEventFlag(LPC_SCT, SCT_EVT_0);
	duty = Chip_SCTPWM_GetDutyCycle(LPC_SCT, 1);

	if (LPC_CMP->CTRL & ACMP_COMPSTAT_BIT) { //Vout > 5V
		if (duty > 1) {
			if (duty == SUC_LIMIT) {
				Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 2);
				stop = 0;
			}
			duty--;
		}
	} else { //Vout < 5V
		if (duty < SUC_LIMIT) {
			duty++;
			if (duty == SUC_LIMIT)
				Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 2);
		} else {
			stop++;
			if (stop > SUC_STOP) {
				Chip_SCTPWM_Stop(LPC_SCT);
				LPC_SCT->OUTPUT = 0;
				while (1);
			}
		}
	}
	if (skip++ % SUC_SKIP)
		return;

	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 1, duty);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO0, PIN_MODE_INACTIVE);   //ACMP_I1
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO3, PIN_MODE_PULLDN);    //GATE
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);    //Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);    //Disable SWDIO
	Chip_SWM_EnableFixedPin(SWM_FIXED_ACMP_I1);    //ACMP_I1
	Chip_SWM_MovablePinAssign(SWM_CTOUT_0_O, 3);    //GATE
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 2);
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 2);

	// Init analog comparator
	Chip_ACMP_Init(LPC_CMP);
	Chip_ACMP_SetPosVoltRef(LPC_CMP, ACMP_POSIN_ACMP_I1);
	Chip_ACMP_SetNegVoltRef(LPC_CMP, ACMP_NEGIN_INT_REF);

	Chip_SCTPWM_Init(LPC_SCT);
	Chip_SCTPWM_SetRate(LPC_SCT, SUC_FREQ);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 1, 1);
	Chip_SCTPWM_SetOutPin(LPC_SCT, 1, 0);
	Chip_SCT_EnableEventInt(LPC_SCT, SCT_EVT_0);
	NVIC_EnableIRQ(SCT_IRQn);
	Chip_SCTPWM_Start(LPC_SCT);

	while (1)
		__WFI();
	return 0;
}
