/*
===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "i2cm.h"
#include "aqm0802a.h"
#include "form.h"
#include "mrt.h"
#include "thermo.h"

// TODO: insert other definitions and declarations here
#define FRQ_ADRS (0x24 << 1)

void alarm(uint8_t adrs, ErrorCode_t err){
	Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 2);
	while(1);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
	SystemCoreClockUpdate();

	//Pin assign
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 0);
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);
 	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	//GPIO setup
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 2);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 3);
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 2);
	Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);

	const uint8_t degree[] = {
		0x07, 0x05, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00
	};

	mrtSetup();
	i2cmSetupErr(alarm);
	i2cmSetup();
	lcdSetup();
	lcdDefChar(3, degree);

	lcdLocate(0, 0);
	lcdPuts("Temperat");
	lcdLocate(0, 1);
	lcdPuts("Wait....");

	uint8_t i2cbuf[5];
	i2cbuf[0] = FRQ_ADRS;
	uint32_t freq;
	int tmp;
	int i;
	while(1){
		mrtWait(MRT_MS(1000));
		Chip_GPIO_SetPinOutHigh(LPC_GPIO_PORT, 0, 3);
		mrtWait(MRT_MS(1000));
		Chip_GPIO_SetPinOutLow(LPC_GPIO_PORT, 0, 3);

		i2cmRx(i2cbuf, 5);
		freq = (i2cbuf[1] << 24) | (i2cbuf[2] << 16) | (i2cbuf[3] << 8) | i2cbuf[4];

		for(i = 0; i < 830 && freq > tmp_hz_tbl[i]; i++);
		tmp = i - 410;

		lcdLocate(0, 1);
		lcdPuts(formDec(tmp, 4, 1));
		lcdDat(3);
		lcdPuts("C)");
	}
    return 0 ;
}
