/*
 ===============================================================================
 Name        : usm810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "mrt.h"
#include "i2cs.h"

// TODO: insert other definitions and declarations here
#define I2C_ADDR 0x29
#define USM_VLADDER 15
#define USM_BURST MRT_US(320)
#define USM_DELAY MRT_US(50)

enum {
	USM_ERR_NEAR = -2,
	USM_ERR_FAR
};

volatile bool timeout;

void usmTimeout(){
	timeout = true;
}

int usmDistance(){
	int count;
	int distance;

	timeout = false;
	mrtOneshot(MRT_MS(100), usmTimeout);//Limit 17m
	Chip_SCTPWM_Start(LPC_SCT);//osc start
	mrtSleep(USM_BURST);
	Chip_SCTPWM_Stop(LPC_SCT);

	mrtSleep(USM_DELAY);
	if(!(LPC_CMP->CTRL & ACMP_COMPSTAT_BIT))
		return USM_ERR_NEAR;

	while(LPC_CMP->CTRL & ACMP_COMPSTAT_BIT)
		if(timeout)
			return USM_ERR_FAR;

	count = MRT_MS(100) - Chip_MRT_GetTimer(LPC_MRT_CH(2));
	distance = (340 / 2) * count / (SystemCoreClock / 1000);//unit mm
	return distance;
}

void changeLad(uint8_t rxn, uint8_t txn){
	if(rxn && (txn == 0)){
		Chip_ACMP_SetupVoltLadder(LPC_CMP, i2csRxBuf[0], false);
		Chip_ACMP_EnableVoltLadder(LPC_CMP);
	}
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO0, PIN_MODE_INACTIVE);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);//Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);//Disable SWDIO
   	Chip_SWM_EnableFixedPin(SWM_FIXED_ACMP_I1);//ACMP_I1
	Chip_SWM_MovablePinAssign(SWM_CTOUT_0_O, 2);
	Chip_SWM_MovablePinAssign(SWM_CTOUT_1_O, 3);
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);//I2C_SCL
	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 4);//I2C_SDA
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_SCTPWM_Init(LPC_SCT);
	Chip_SCTPWM_SetRate(LPC_SCT, 40000);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 1, Chip_SCTPWM_PercentageToTicks(LPC_SCT, 50));
	Chip_SCTPWM_SetOutPin(LPC_SCT, 1, 0);
	LPC_SCT->OUT[1].SET = 1 << 1;
	LPC_SCT->OUT[1].CLR = 1;

	//MRT setup
	mrtSetup();
	mrtSetupSleep();

	//ACMP setup
	Chip_ACMP_Init(LPC_CMP);
	Chip_ACMP_SetupVoltLadder(LPC_CMP, USM_VLADDER, false);
	Chip_ACMP_EnableVoltLadder(LPC_CMP);
	mrtSleep(MRT_US(30));//30us power-up settling time

	Chip_ACMP_SetPosVoltRef(LPC_CMP, ACMP_POSIN_VLO);
	Chip_ACMP_SetNegVoltRef(LPC_CMP, ACMP_NEGIN_ACMP_I1);
	Chip_ACMP_SetHysteresis(LPC_CMP, ACMP_HYS_10MV);

	i2csSetupCallback(changeLad);
	i2csSetup(I2C_ADDR, 1, 2);

	int distance;

	while (1) {
		distance = usmDistance();
		__disable_irq();
		i2csTxBuf[0] = distance >> 8;
		i2csTxBuf[1] = distance & 0xff;
		__enable_irq();

		mrtSleep(MRT_MS(1000));
	}
	return 0;
}
